#include "aimmsLink.h"

#include <iostream>
#include <map>
#include "ErrorCollector.h"
#include "Project.h"
#include "aimms/AimmsMessage.h"


std::map<int, aimms::Project*> Projects;
int Handle;
aimms::ErrorCollector* ErrorStack;

void InitializeDynamicLibrary()
{
    std::cout << "loaded AIMMS Link" << std::endl;
    ErrorStack = new aimms::ErrorCollector;    
    Handle = 0;
}

void FinalizeDynamicLibrary() 
{
    std::cout << "unloaded AIMMS Link" << std::endl;
    if ( Projects.size() > 0 ){
        std::map<int, aimms::Project*>::iterator it = Projects.begin();
        std::map<int, aimms::Project*>::iterator end = Projects.end();
        while(it != end){
            aimms::Project* project = (*it).second;

            std::cerr << "error: AIMMS project '" << project->getProjectName() << "' is still open, discarding" << std::endl;

            ++it;
        }
    }
    if (ErrorStack ){
        while (!ErrorStack->m_ErrorStack.empty()){
            std::cerr << "unhandled error: " <<  ErrorStack->m_ErrorStack.top() << std::endl;
            ErrorStack->m_ErrorStack.pop();
        }
        delete ErrorStack;
        ErrorStack = 0;
    }
}


int AimmsOpenProject(const char* aimmsFolder,  const char* projectName,int* projectHandle, int timeOut, int maxFlushInBackground, int maxMemoryFlushInBackground, int maxBufferSize)
{
    aimms::Project* project = new aimms::Project;

    int ret = project->open(aimmsFolder, projectName, timeOut, maxFlushInBackground, maxMemoryFlushInBackground, maxBufferSize, ErrorStack);
    if ( ret == 0 ){
        *projectHandle = ++Handle;
        Projects[Handle] = project;
    } else {
        project->close();
    }
    return ret;
}

aimms::Project* getExistingProject(int projectHandle){
    std::map<int, aimms::Project*>::const_iterator it = Projects.find(projectHandle);
    if (it != Projects.end()){
        return (*it).second;
    }
    return 0;
}

int AimmsCloseProject(int projectHandle)
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }
    Projects.erase(projectHandle);
    return project->close();
}

int AimmsAssignSet(int projectHandle,  const char* setName,  int card,   const char* const* labels, bool clearFirst )
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }

    return project->assignSet(setName, card, labels, clearFirst);
}

int AimmsIntAssignIdentifier(int projectHandle, const char* identifierName, int dimension, int* firsts, int* lasts, float* identifier)
{
    return AimmsIntAssignIdentifierSlice(projectHandle,identifierName,dimension,firsts,lasts,identifier,0,0);
}

int AimmsIntAssignIdentifierSlice(int projectHandle, const char* identifierName, int dimension, int* firsts, int* lasts, float* identifier, int** slices, int* slicecardinalities)
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }

    return project->assignIntIdentifier(identifierName, dimension, firsts, lasts, identifier,  slices, slicecardinalities);
}

int AimmsIntRetrieveIdentifier(int projectHandle, const char* identifierName, int dimension, int* firsts, int* lasts, float* identifier)
{
    return AimmsIntRetrieveIdentifierSlice(projectHandle,identifierName,dimension,firsts,lasts,identifier,0,0);
}


int AimmsIntRetrieveIdentifierSlice(int projectHandle, const char* identifierName, int dimension, int* firsts, int* lasts, float* identifier, int** slices, int* slicecardinalities)
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }

    return project->retrieveIntIdentifier(identifierName, dimension, firsts, lasts, identifier,  slices, slicecardinalities);
}

int AimmsOrdAssignIdentifier(int projectHandle, const char* identifierName, int dimension, int* cards, float* identifier)
{
    return AimmsOrdAssignIdentifierSlice(projectHandle,identifierName,dimension,cards,identifier,0,0);
}

int AimmsOrdAssignIdentifierSlice(int projectHandle, const char* identifierName, int dimension, int* cards, float* identifier, int** slices, int* slicecardinalities)
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }

    return project->assignOrdIdentifier(identifierName, dimension, cards, identifier, slices, slicecardinalities);
}

int AimmsOrdRetrieveIdentifier(int projectHandle, const char* identifierName, int dimension, int* cards, float* identifier)
{
    return AimmsOrdRetrieveIdentifierSlice(projectHandle,identifierName,dimension,cards,identifier,0,0);
}


int AimmsOrdRetrieveIdentifierSlice(int projectHandle, const char* identifierName, int dimension, int* cards, float* identifier, int** slices, int* slicecardinalities)
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }
    return project->retrieveOrdIdentifier(identifierName,dimension, cards, identifier, slices, slicecardinalities);
}

int AimmsIntAssignIdentifierDouble(int projectHandle, const char* identifierName, int dimension, int* firsts, int* lasts, double* identifier)
{
    return AimmsIntAssignIdentifierSliceDouble(projectHandle,identifierName,dimension,firsts,lasts,identifier,0,0);
}

int AimmsIntAssignIdentifierSliceDouble(int projectHandle, const char* identifierName, int dimension, int* firsts, int* lasts, double* identifier, int** slices, int* slicecardinalities)
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }

    return project->assignIntIdentifier(identifierName, dimension, firsts, lasts, identifier,  slices, slicecardinalities);
}

int AimmsIntRetrieveIdentifierDouble(int projectHandle, const char* identifierName, int dimension, int* firsts, int* lasts, double* identifier)
{
    return AimmsIntRetrieveIdentifierSliceDouble(projectHandle,identifierName,dimension,firsts,lasts,identifier,0,0);
}

int AimmsIntRetrieveIdentifierSliceDouble(int projectHandle, const char* identifierName, int dimension, int* firsts, int* lasts, double* identifier, int** slices, int* slicecardinalities)
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }

    return project->retrieveIntIdentifier(identifierName, dimension, firsts, lasts, identifier,  slices, slicecardinalities);
}

int AimmsOrdAssignIdentifierDouble(int projectHandle, const char* identifierName, int dimension, int* cards, double* identifier)
{
    return AimmsOrdAssignIdentifierSliceDouble(projectHandle,identifierName,dimension,cards,identifier,0,0);
}

int AimmsOrdAssignIdentifierSliceDouble(int projectHandle, const char* identifierName, int dimension, int* cards, double* identifier, int** slices, int* slicecardinalities)
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }

    return project->assignOrdIdentifier(identifierName, dimension, cards, identifier, slices, slicecardinalities);
}

int AimmsOrdRetrieveIdentifierDouble(int projectHandle, const char* identifierName, int dimension, int* cards, double* identifier)
{
    return AimmsOrdRetrieveIdentifierSliceDouble(projectHandle,identifierName,dimension,cards,identifier,0,0);
}

int AimmsOrdRetrieveIdentifierSliceDouble(int projectHandle, const char* identifierName, int dimension, int* cards, double* identifier, int** slices, int* slicecardinalities)
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }
    return project->retrieveOrdIdentifier(identifierName,dimension, cards, identifier, slices, slicecardinalities);
}



int AimmsGetLastErrorMessage( int* textBufferLen, char* textBuffer )
{
    return ErrorStack->getLastError(textBufferLen, textBuffer);
}

int AimmsRunProcedure( int projectHandle, const char* procedureName, int* returnCode )
{
    aimms::Project* project = getExistingProject(projectHandle);
    if(!project){
        ErrorStack->addError("Not a valid projecthandle");
        return aimms::AM_InvalidUse;
    }

    return project->runProcedure(procedureName, *returnCode);
}